import { ApiProperty, OmitType, PickType } from '@nestjs/swagger';
import { Type } from 'class-transformer';
import { Building } from 'src/objects/building/entities/building.entity';
import { Floor } from 'src/objects/building/entities/floor.entity';
import { Room } from 'src/objects/building/entities/room.entity';

export class BuildingMinimalFloorResponseDto extends PickType(Floor, [
  'id',
  'number',
  'displayName',
]) {}

export class BuildingRoomWithFloorResponseDto extends OmitType(Room, [
  'floor',
  'building',
]) {
  @ApiProperty({ type: BuildingMinimalFloorResponseDto })
  @Type(() => BuildingMinimalFloorResponseDto)
  floor: BuildingMinimalFloorResponseDto;
}

export class BuildingRoomResponseDto extends OmitType(Room, [
  'floor',
  'building',
]) {}

export class BuildingFloorResponseDto extends OmitType(Floor, [
  'building',
  'rooms',
]) {
  @ApiProperty({ type: BuildingRoomResponseDto, isArray: true })
  @Type(() => BuildingRoomResponseDto)
  rooms: BuildingRoomResponseDto[];
}

export class BuildingsResponseDto extends OmitType(Building, [
  'groups',
  'floors',
]) {
  @ApiProperty({ type: BuildingFloorResponseDto, isArray: true })
  @Type(() => BuildingFloorResponseDto)
  floors: BuildingFloorResponseDto[];
}

export class BuildingsListResponseDto extends OmitType(BuildingsResponseDto, [
  'floors',
]) {}
