import { ApiProperty, ApiPropertyOptional } from '@nestjs/swagger';
import { User } from 'src/objects/user/user.entity';
import {
  Column,
  CreateDateColumn,
  Entity,
  ManyToOne,
  PrimaryGeneratedColumn,
  UpdateDateColumn,
} from 'typeorm';
import { TransactionType } from '../enums/transaction-type.enum';
import { StoredItem } from './stored-item.entity';

@Entity()
export class StoredItemTransaction {
  @ApiProperty()
  @PrimaryGeneratedColumn()
  id: number;

  @ApiProperty({ type: String, enum: TransactionType })
  @Column({ type: String, default: TransactionType.ACQUIRED })
  type: TransactionType;

  @ApiPropertyOptional()
  @Column({ nullable: true })
  price?: number;

  @ApiPropertyOptional()
  @Column({ nullable: true, default: 'EUR', length: 3 })
  currency?: string;

  @ApiProperty({ type: () => StoredItem })
  @ManyToOne(() => StoredItem, {
    onDelete: 'CASCADE',
    onUpdate: 'CASCADE',
  })
  storedItem: StoredItem;

  @ApiPropertyOptional({ type: () => User })
  @ManyToOne(() => User, {
    onDelete: 'SET NULL',
    nullable: true,
  })
  actor?: User;

  @ApiPropertyOptional()
  @Column({ nullable: true })
  notes?: string;

  @ApiPropertyOptional()
  @Column({ type: 'datetime', nullable: true })
  actionAt?: Date;

  @ApiProperty()
  @CreateDateColumn()
  createdAt: Date;

  @ApiProperty()
  @UpdateDateColumn()
  updatedAt: Date;
}
