import { Injectable, CanActivate, ExecutionContext } from '@nestjs/common';
import { Request } from 'express';
import { BuildingService } from 'src/objects/building/building.service';
import { User } from 'src/objects/user/user.entity';

@Injectable()
export class BuildingGuard implements CanActivate {
  constructor(private readonly buildingService: BuildingService) {}

  async canActivate(context: ExecutionContext): Promise<boolean> {
    const http = context.switchToHttp();
    const request = http.getRequest() as Request;
    const response = http.getResponse();

    const user = response.locals.user as User;
    if (!user) return false;

    if (
      request.params.buildingId == null &&
      request.body?.buildingId == null &&
      request.query?.buildingId == null
    ) {
      return true;
    }

    const buildingId = parseInt(
      request.params.buildingId ||
        request.body?.buildingId ||
        request.query?.buildingId,
      10,
    );

    if (!buildingId || isNaN(buildingId)) return false;

    const buildingAccess = await this.buildingService.getBuildingByIdAndUserSub(
      buildingId,
      user.sub,
    );
    if (!buildingAccess) return false;
    response.locals.building = buildingAccess;

    return true;
  }
}
